#!/usr/bin/env python

"""
CourseBuilder 

Coursebuilder is a preprocessor tool for [pandoc](https://pandoc.org) 
to generate multi-lingual curricula documentation tables from 
structured representations as a flatfile database. Data scheme and 
actual values are kept in YAML files in order to version them with git.

"""

from argparse import ArgumentParser
import string
import os,sys
import yaml
import pandas as pd


from tablegenerator import TableGenerator
from markdowngenerator import MarkdownGenerator
from templategenerator import TemplateGenerator
from parser import Parser

class CourseBuilder:

    @staticmethod
    def run():
        
        # arguments
        parser = ArgumentParser(description='versatile curricula generator')

        parser.add_argument('-m','--meta',action="extend", nargs="+", type=str,help="course description(s) as YAML file(s)")
        parser.add_argument('-l','--lang',help="Language to parse from meta file (use de or en)",default='de')
        parser.add_argument('-f','--fields',help="Fields to be used, the table will be build accordingly",action="extend", nargs="+", type=str)
        parser.add_argument('-s','--schema',help="using provided schema")
        parser.add_argument('-p','--pagebreak',action="store_true",help="add a pagebreak after each module")
        parser.add_argument('--title',type=str,default=None,help="template for title")
        parser.add_argument('-b','--book',type=str,help="process a whole curriculum book with sections")
        parser.add_argument('--level',type=int,default=1,help="level of header tags")
        parser.add_argument('--table-gen',type=str,default=None,help='runs table generator')
        parser.add_argument('--template',type=str,default=None,help='defines a template to be used with fields')
        parser.add_argument('-o','--out',type=str,default=None,help='set the output type')

        parser.add_argument('--maxcol',type=int,default=28,help='maximum size of left column')

        # get arguments
        args = parser.parse_args()
        
        if args.table_gen:

            tg = TableGenerator()
            
            tg.generate_table(args.table_gen)

            return


        # book mode with predefined setting from a book file
        if args.book and args.schema:
            
            generator = Parser()

            with open(args.schema) as sf:
                generator.set_schema(yaml.load(sf,Loader=yaml.Loader))

            with open(args.book) as bf:

                actual_fields = []

                book = yaml.load(bf,Loader=yaml.Loader)
                book_path = os.path.abspath(args.book)

                for bi in book['book']:
                    if 'fields' in bi:
                        actual_fields = bi['fields']
                    if 'sections' in bi:
                        for section in bi['sections']:
                            if 'text' in section:
                                print(section['text'][args.lang])
                            if 'modules' in section:
                                for m in section['modules']: 
                                    mod_path = os.path.join(os.path.dirname(book_path),m)

                                    with open(mod_path) as fm:
                                        try:

                                            table_items = generator.process(yaml.load(fm,Loader=yaml.Loader),fields=actual_fields,lang=args.lang,pagebreak=args.pagebreak,createTitle=args.title,header_level=args.level)
                                            
                                            MarkdownGenerator.generate(table_items,pagebreak=args.pagebreak,title=args.title,header_level=args.level)
                                            
                                        except Exception as exc:
                                            print(f'{type(exc).__name__} in {mod_path}: {exc}',file=sys.stderr)


        # verbose command line mode
        elif args.schema and args.meta and len(args.fields) > 0:
            
            # get actual fields
            actual_fields = []

            if os.path.isfile(args.fields[0]):
                with open(args.fields[0]) as ff:
                    actual_fields = yaml.load(ff,Loader=yaml.Loader)['fields']
            else:
                actual_fields = args.fields 
            
        
            # get schema
            actual_schema = None
            with open(args.schema) as f:
                actual_schema = yaml.load(f,Loader=yaml.Loader)

            # iterate through meta files
            for m in args.meta:
                with open(m) as fm:
        
                    generator = Parser()
                    generator.set_schema(actual_schema)

                    meta = yaml.load(fm,Loader=yaml.Loader)

                    table_items = generator.process(meta=meta,fields=actual_fields,lang=args.lang)

                    # TODO - something more processable for Pandas?
                    # return [ { name: Computergraphik  }, keys = { name: Modulname } ]

                    # get the dataframe
                    df = pd.DataFrame(table_items)

                    # use first column for 
                    df.columns = df.iloc[0]
                    df = df[1:]

                    if args.title != None:
                        print(args.title.format(df.columns[1]),'\n')

                    print(df.to_markdown(tablefmt='grid', index=False, maxcolwidths=[args.maxcol,None]))
                    print('\n') # always add a newline after the table

                    if args.pagebreak:
                        print('\\pagebreak')


                        # MarkdownGenerator.generate(table_items,pagebreak=args.pagebreak,title=args.title,header_level=args.level)

        else:
            parser.print_help()

if __name__ == '__main__':
    CourseBuilder.run()
